/*
 *				**** Area Code ****
 *
 * This program finds area code information
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "prodlist.h"
#include "Linklist.h"

struct areacode {
	char *areacode;
   char *state;
   char *desc;
};

static int debugsw=0;

/* split fields prototype */
int split_flds(char *str,char **flds, char delim,int skip, int maxflds);

LinkList *ingest_areacode(char *fname);
int read_line(char *buf, FILE *fp); /* non-zero is error */
void display_areacode(struct areacode *ac);

int main(int argc, char **argv)
{
	ProdAPtrList *areacode, *state;
   ProdPtrList *ptr;
   LinkList *AreaCodes;
   Link *link;
   char buf[BUFSIZ];	/* input buffer */
   char *s;				/* string pointer */
   char *p;
   /* storage space for areacode structures */
   struct areacode *ac, *actemp;
   int loc, i;

   /* create linked list for all entries */
   AreaCodes = ingest_areacode("areacode.txt");
   if (AreaCodes == NULL)
   {
   	fprintf(stderr, "Can't Read input file!\n");
      exit(1);
   }

   /* create index structures for state & areacode */
   areacode = create_aptr_prodlist(100);	/* 100 entries */
   state = create_aptr_prodlist(100);	/* areacode & state */

   for (set_head_current_link(AreaCodes);	/* start at front of list */
   	(ac = (struct areacode *)get_data_from_current(AreaCodes)) != NULL;
     set_current_link_next(AreaCodes))
   { /* loop thru linked list & index entries */
   	if (debugsw > 5)
      {
	   	printf("Building index for: ");
	      display_areacode(ac);
      }
   	/* now add to areacode & state indecies */

      /* area code must be unique */
     	actemp = (struct areacode *)bupdate_aptr_prodlist(areacode, ac->areacode, ac);
     	if (actemp != NULL)
   	{ /* already present - should never happen */
     		fprintf(stderr, "Duplicate Area Code\n");
         display_areacode(actemp);
         free(actemp);
    	}

      /* there can be multiple entries for a state */
      /* see if entry already there */
      loc = bsearch_aptr_prodlist(state, ac->state);
      if (loc < 0)
      { /* first time for state */
			ptr = create_ptr_prodlist(10); /* acs for state */
         if (ptr == NULL)
         {
         	fprintf(stderr, "Out of memory!\n");
            exit(1);
         }
         bupdate_aptr_prodlist(state, ac->state, (void *)ptr);
         loc = bsearch_aptr_prodlist(state, ac->state);
         if (loc < 0)
         {
         	fprintf(stderr, "Internal error - can't find state entry!\n");
            exit(2);
         }
      }
      /* state entry is in position loc */
      ptr = (ProdPtrList *)state->item[loc];
      add_ptr_prodlist(ptr, ac);	/* add areacode to state */

      if (current_is_tail(AreaCodes)) break;	/* end of list */
	}

   if (debugsw > 0)
   { /* display keys */
   	for (i=0; i < areacode->nitems; i++)
      { /* loop thru areacodes */
      	printf("ac[%d]\t%s\n", i, areacode->key[i]);
      }

      for (i=0; i < state->nitems; i++)
      { /* loop thru states */
      	printf("state[%d]\t%s\n", i, state->key[i]);
      }
   }

   /* main loop */
   while (1)
   {
		printf("Enter areacode, state, or ENTER to quit: ");
      gets(buf);
      if (buf[0] == '\0') break;
      if (isdigit(*buf))
      { /* digit entered - must be area code */
	      loc = bsearch_aptr_prodlist(areacode, buf);
   	   if (loc >= 0)
         {
         	ac = (struct areacode *)areacode->item[loc];
         	if (ac != NULL) display_areacode(ac);
            else fprintf(stderr, "Internal error - can't find areacode!\n");
         }
         else printf("Areacode Unknown!\n");
      } else if (isalpha(*buf))
      { /* alphabetic - should be state */
      	/* state is in upper case - convert input */
         for (i=0; buf[i] != '\0'; i++)
         {
         	if (islower(buf[i])) buf[i] = toupper(buf[i]);
         }
      	loc = bsearch_aptr_prodlist(state, buf);
         if (loc < 0)
         {
         	printf("Areacode unknown\n");
         }
         else
         {
         	ptr = (ProdPtrList *)state->item[loc];
            for (i=0; i < ptr->nitems; i++)
            { /* loop thru state entries */
            	ac = (struct areacode *)ptr->item[i];
            	display_areacode(ac);
            }
         }
      }
   }
   printf("Press ENTER to terminate: ");
   gets(buf);
   return 0;
}

LinkList *
ingest_areacode(char *fname)
{
	FILE *fp;			/* file pointer */
   /* fields for splitfields */
   int nflds;
   char *flds[5];	/* storage for fields */
   struct areacode *ac;
   char buf[BUFSIZ], *s;
   LinkList *ret;
   Link *link;

   /*
    * assume a file areacode.txt contains data
    * with the following format:
    *		areacode|state|description
    *
    * For example:
    *		215|PA|Philadelphia|
    *		610|PA|West of Philadelphia|
    */
    if ((fp = fopen(fname, "r")) == NULL)
    {
    	fprintf(stderr, "Can't open input file (%s)\n", fname);
      return NULL;	/* error return */
    }

    ret = create_link_list();

    while (read_line(buf, fp) == 0)
    { /* read lines from file */

      /*
       * split_flds modifies the string it is passed
       * make a copy for use in structure
       */
      s = strdup(buf);	/* make copy */
      nflds = split_flds(s, flds, '|', 0, 5); /* parse line */
      if (nflds < 3 || nflds > 4)
      { /* bad line format */
			fprintf(stderr,  "Invalid line >>%s<< (%d fields instead of 4)\n",
         	buf, nflds);
         free(s);	/* free string */
         continue;
      }

      /* valid line - create structure & store info */
      ac = (struct areacode *)malloc(sizeof(struct areacode));
      if (ac == NULL) break;	/* out of memory */
		ac->areacode = strdup(flds[0]);
      ac->state = strdup(flds[1]);
      ac->desc = strdup(flds[2]);

      /* add structure to linked list */
      link = create_link_entry(ac);
      if (link == NULL) break; /* out of memory */
      add_link_to_tail(ret, link); /* append to link */
      if (debugsw > 5)
      {
	      printf("Added to list: ");
	      display_areacode(ac);
      }
    }
    return ret;
}

int
read_line(char *buf, FILE *fp)
{
	char *p;

   if (buf == NULL || fp == NULL) return -1; /* bad args */

	if (fgets(buf, BUFSIZ, fp) == NULL) return 1;

   /* strip CR-LF from line */
   for (p=buf; *p != '\0' && *p != '\r' && *p != '\n'; p++);
   *p = '\0';	/* terminate string */

	return 0;
}

void
display_areacode(struct areacode *ac)
{
	if (ac == NULL) return;
	printf("Areacode %s, State %s, %s\n", ac->areacode, ac->state, ac->desc);
}


